import time
from subprocess import Popen, PIPE

from src.ha_handler.ha_handler import (is_full_ha_mode, check_full_ha_enable_pme, get_redundancy_nodelist)
from src.cli_commons.commons_manager import (installed_application, show_select_option)
from network_management import reach_ip
import click

import configparser
mapper = configparser.RawConfigParser()
mapper.read('mapper.ini')

logs_path = configparser.RawConfigParser()
logs_path.read('logs.ini')

def beanstalkd_stop():
    click.echo('Stopping...')
    p = Popen(["sh", mapper.get('GeneralScripts', 'beanstalkdStop')], stdout=PIPE, stderr=PIPE)
    for line in p.stdout:
        line = line.decode("utf-8")
        print(line)
    p.wait()

    if p.returncode != 0:
        click.secho('[FAIL] Beanstalkd Could not be Stopped', fg="red")
    else:
        click.secho('[OK] Beanstalkd Stopped Successfully', fg="yellow")

    time.sleep(3)


def beanstalkd_start():
    click.echo('Starting...')
    p = Popen(["sh", mapper.get('GeneralScripts', 'beanstalkdStart')], stdout=PIPE, stderr=PIPE)
    for line in p.stdout:
        line = line.decode("utf-8")
        print(line)
    p.wait()

    if p.returncode != 0:
        click.secho('[FAIL] Beanstalkd Could not be Started', fg="red")
    else:
        click.secho('[OK] Beanstalkd Started Successfully', fg="yellow")

    time.sleep(3)


def beanstalkd_restart():
    ha_enabled = check_full_ha_enable_pme()
    if ha_enabled:
        check = click.prompt("Do you wish to Restart Beanstalkd on all reachable nodes? (y or N)\nThis will affect APM Provisioning so please make sure there are no jobs running")
        if check == 'y' or check == 'Y':
            click.echo('Restarting all nodes...')
            node_list = get_redundancy_nodelist()
            nodes_list_array = node_list.split(' ')
            for i in range(0, len(nodes_list_array)):
                if reach_ip(nodes_list_array[i]):
                    node = nodes_list_array[i]
                    time.sleep(3)
                    try:
                        Popen(["sh", mapper.get('GeneralScripts', 'beanstalkdRestartViaSSH'), node], stdout=PIPE, stderr=PIPE)
                        click.secho('[OK] Beanstalkd Restarted Successfully on Node' + node, fg="yellow")
                    except Exception as e:
                        click.secho('[FAIL] Beanstalkd Could not be Restarted on node' + node, fg="red")
                        click.secho('ERROR:' + str(e), fg="red")
                    time.sleep(3)
        else:
            time.sleep(3)
            click.pause('Please, press any key to continue...')
    else:
        time.sleep(3)
        beanstalkd_restart_current_node()

def beanstalkd_restart_current_node():
    click.echo('Restarting...')
    p = Popen(["sh", mapper.get('GeneralScripts', 'beanstalkdRestart')], stdout=PIPE, stderr=PIPE)
    for line in p.stdout:
        line = line.decode("utf-8")
        print(line)
    p.wait()

    if p.returncode != 0:
        click.secho('[FAIL] Beanstalkd Could not be restarted', fg="red")
    else:
        click.secho('[OK] Beanstalkd Restarted Successfully', fg="yellow")

def beanstalkd_status():
    p = Popen(["sh", mapper.get('GeneralScripts', 'beanstalkdStatus')], stdout=PIPE, stderr=PIPE)
    for line in p.stdout:
        line = line.decode("utf-8")
        print(line)
    p.wait()

    click.pause('Please, press any key to continue...')