import base64
import logging
import socket
from subprocess import Popen, PIPE

import mapper_python_scripts

logging.basicConfig(filename='executor.log', level=logging.DEBUG)

logging.debug('about logging mapper.ini')

# This handle the python2 and python3 versions
try:
    import configparser

    mapper = configparser.RawConfigParser()
    mapper.read('mapper.ini')
except:
    import ConfigParser

    mapper = ConfigParser.RawConfigParser()
    mapper.read('mapper.ini')

# Also read mapper for python scripts.
mapper.read_dict(mapper_python_scripts.pyScripts)

HOST = '0.0.0.0'  # In this way we ensure nothing can connect from outside. 0.0.0.0 for any ip
PORT = 65432  # Port to listen, maybe there is a more secure port?

s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
s.bind((HOST, PORT))
s.listen(5)
while 1:
    (conn, addr) = s.accept()
    logging.debug('Connected by ' + str(addr))
    data = conn.recv(1024).decode("utf-8")
    if not data:
        logging.debug('There is no data in the message')
        continue
    logging.debug('Data received ' + data)

    # example of message: isValidUserPass, GeneralScripts:isValidUserPass
    function_and_parameters = data.split(":")
    is_needed_wait = True
    script = ""
    section, function, params = "", "", []

    #  if data with valid section or valid option.
    if mapper.has_section(function_and_parameters[0]) \
            or mapper.has_option("GeneralScripts", function_and_parameters[0]) \
            or mapper.has_option("PythonScripts", function_and_parameters[0]):

        # it has section
        is_section_present = mapper.has_section(function_and_parameters[0])
        is_general_scripts = mapper.has_option('GeneralScripts', function_and_parameters[0])

        section = function_and_parameters[0] if is_section_present \
            else 'GeneralScripts' if is_general_scripts \
            else 'PythonScripts'

        function = function_and_parameters[1] if is_section_present else function_and_parameters[0]

        params = function_and_parameters[2:] if is_section_present else function_and_parameters[1:]

        if str.endswith(function, "Async"):
            function = function.rstrip("Async")
            is_needed_wait = False
        try:
            script = mapper.get(section, function)
        except configparser.NoSectionError:
            logging.error('No section or function found with data: ' + data)

    logging.debug('mapper to use: ' + script)

    response = ''

    if section == 'PythonScripts' and script != '':
        script = getattr(mapper_python_scripts, script)
        for i in range(len(params)):
            params[i] = base64.b64decode(params[i]).decode('utf-8')

        # This functions must take an array as parameters
        result = script(params)

        response = str(result)
    elif script != '':
        if function == 'acoUpdateScript':
            # cucm url is endoded
            params[1] = base64.b64decode(params[1])

        p = Popen(["sh", script] + params, stdout=PIPE, stderr=PIPE)

        if is_needed_wait:
            p.wait()

            for line in p.stdout:
                line = line.decode("utf-8")
                logging.debug(line)
                response = response + line

            for line in p.stderr:
                line = line.decode("utf-8")
                logging.debug(line)
                response = response + line

            logging.debug('Response is: ' + response)
        else:
            response = function + " triggered successfully"

    conn.send((response + "\r\n").encode("utf-8"))
