import logging
import time
from src.ha_handler.ha_handler import get_redundancy_nodelist, installed_application, reach_ip
from MariaDBAndGaleraHelper import MariadbGaleraHelper
from email.message import EmailMessage
import smtplib
import ssl
from subprocess import Popen, PIPE
import configparser

mapper = configparser.RawConfigParser()
mapper.read('mapper.ini')


logconfig = configparser.RawConfigParser()
logconfig.read('/home/akkadianuser/logs-config.ini')
loglevel = logconfig.get('General', 'galera-replication')
log_level = int(loglevel)

logging.basicConfig(format='%(asctime)s %(levelname)-8s %(message)s', filename='/home/akkadianuser/logs/galera_failover_recovery.log', level=log_level, datefmt='%Y-%m-%d %H:%M:%S')
credentialsconfig = configparser.RawConfigParser()
credentialsconfig.read('/home/akkadianuser/.my.cnf')
mariadb_user = credentialsconfig.get('client', 'user')
mariadb_password = credentialsconfig.get('client', 'password')

SEND_EMAIL_TIME = 60

class ReplicaFailover:

    def __init__(self):
        """ self.__email_sch = sched.scheduler(time.time, time.sleep)
        self.__alreadySet = False
        self.__email_t = None """
        self.__shouldIRun = True
        logging.debug("Super class")


    def set_should_I_run(self,b):
        self.__shouldIRun = b


    def connected_made_on_client(self):
        logging.debug('connection made on client, checking mariadb status')
        mariadb_recover = MariadbGaleraHelper(mariadb_user, mariadb_password)
        data_received = mariadb_recover.statusMariadb()

        logging.debug('data_received: ' + data_received[0])
        while data_received[0].find("Active: failed") != -1 or data_received[0].find("Active: inactive (dead)") != -1:
            logging.debug('mariadb in client is down')
            time.sleep(15)
            mariadb_recover.startMariadb()
            time.sleep(3)
            data_received = mariadb_recover.statusMariadb()


    def connected_made_on_server(self, host):
        logging.debug('connection made on server, checking mariadb status')
        mariadb_recover = MariadbGaleraHelper(mariadb_user, mariadb_password)
        mariadb_recover.checkStatusAndIfNeededRestartClusterWhenAllNodesUp(host)

        #When server starts and replica-failover is enabled, this will call flush queue method. Method will call on a shell script.
        self.run_flush_queue()

    def run_flush_queue(self):
        p = Popen(["sh", mapper.get('GeneralScripts', 'flushQueue')], stdout=PIPE, stderr=PIPE)
        for line in p.stdout:
            line = line.decode("utf-8")
            print(line)
        p.wait()

    def recover_mariadb_one_node(self):
        logging.debug('recovering one node')
        mariadb_recover = MariadbGaleraHelper(mariadb_user, mariadb_password)

        mariadb_recover.forceBootstap()
        mariadb_recover.killMariadb()
        mariadb_recover.bootstrapMariadb()

        time.sleep(15)
        data_received = mariadb_recover.statusMariadb()
        if data_received[0].find("Active: active (running)") == -1:
            mariadb_recover.killMariadb()
            mariadb_recover.bootstrapMariadb()

    def send_email(self, scheduler):
        email_config = False
        configs = configparser.RawConfigParser()
        try:
            configs.read('/home/akkadianuser/email_configs.ini')
            email_config = True
        except:
            email_config = False

        if email_config:

            send_email = configs.get('General', 'enable')
            if send_email:
                msg = EmailMessage()

                node_list = get_redundancy_nodelist()
                offline = ""
                online = ""

                nodes_list_array = node_list.split(' ')

                for i in range(0, len(nodes_list_array)):
                    if reach_ip(nodes_list_array[i]):
                        online = online + " " + nodes_list_array[i]
                    else:
                        offline = offline + " " + nodes_list_array[i]

                # GET THE NAME OF THE NODE DOWN.
                message = "HA Cluster connectivity to " + offline + " has been lost, please verify cluster status in Akkadian Appliance Manager"

                msg.set_content(message)

                installed_app = installed_application()
                name_to_show = ""
                if installed_app == 'aco':
                    name_to_show = "Akkadian Console Operator"
                elif installed_app == 'pme_cm':
                    name_to_show = 'Akkadian Provisioning Manager'
                elif installed_app == 'acm':
                    name_to_show = 'Akkadian Contact Manager'

                msg['Subject'] = f'[ALERT] Server status: DOWN - ' + name_to_show

                msg['From'] = configs.get('General', 'sender')
                msg['To'] = configs.get('General', 'receiver')

                port = configs.get('General', 'port')  # For SSL
                password = configs.get('General', 'password')

                ssl_ = configs.get('General', 'is_ssl_enable')
                # Create a secure SSL context

                if ssl_ == "True":
                    context = ssl.create_default_context()
                    context.check_hostname = False
                    context.verify_mode = ssl.CERT_NONE

                sender_email = configs.get('General', 'sender')
                receiver_email = configs.get('General', 'receiver')
                sever_add = configs.get('General', 'server')

                port_int = int(port)
                try:
                    with smtplib.SMTP(sever_add, port_int) as server:
                        if ssl_ == "True":
                            server.ehlo()
                            server.starttls(context=context)
                            server.ehlo()
                            server.login(sender_email, password)
                            server.send_message(msg)
                        else:
                            if password != "":
                                server.login(sender_email, password)
                            server.send_message(msg)

                        server.quit()
                        logging.debug("Email Sent")
                except Exception as e:
                    logging.debug("There was a problem in the email configuration, please check")

        if self.__shouldIRun:
            delay = configs.get('General', 'delay')
            delay_float = float(delay) * 60
            scheduler.enter(delay_float, 1, self.send_email, (self, scheduler))
