#!/usr/bin/env bash

# @dependency /usr/bin/jq
jq_rpm_path=/home/akkadianuser/dependencies/jq-1.6-2.el7.x86_64.rpm
# If jq is installed:
if sudo rpm -q jq &>/dev/null; then
  # If jq binary size is not greater than 0:
  if [[ ! -s /usr/bin/jq ]]; then
    sudo rpm -i --force ${jq_rpm_path}
  fi
fi

# If jq is not installed:
if ! sudo rpm -q jq &>/dev/null; then
  # Since both yum attempts to install jq failed, use the included rpm.
  sudo rpm -i --force ${jq_rpm_path}
fi

if [[ ! -d /var/www ]]; then
  sudo mkdir /var/www
  sudo chown -R apache:root /var/www
  sudo chmod -R 755 /var/www
fi

if [[ ! -d /var/www/product_update ]]; then
  sudo mkdir /var/www/product_update
  sudo chown -R apache:root /var/www/product_update
  sudo chmod -R 755 /var/www/product_update
fi

sudo cp -Rf /home/akkadianuser/scripts/universal_updater/update_main.sh /var/www/product_update/

if [[ ! -d /var/www/html/ha ]]; then
  sudo mkdir /var/www/html/ha
  sudo chown -R apache:root /var/www/html/ha
  sudo chmod -R 755 /var/www/html/ha
fi

if [[ ! -d /var/www/cli_updates ]]; then
  sudo mkdir /var/www/cli_updates
  sudo chown -R apache:root /var/www/cli_updates
  sudo chmod -R 755 /var/www/cli_updates
fi

if [[ -d /var/www/html/asb ]]; then
  if [[ ! -f /home/akkadianuser/scripts/executed_fixes/asb_1x_update_fix.done ]]; then
    sudo sh /home/akkadianuser/scripts/asb/asb_1x_update_fix.sh /var/www/html/asb
  fi
fi

#if [[ ! -d /usr/local/sbin/asm ]]; then
# echo "Running Akkadian Service Monitor installation" >> /var/www/product_update/update.log
#sh /home/akkadianuser/upgrades/akkadian_service_monitor/install_asm.sh >> /var/www/product_update/update.log
#fi

# Create hcs directory if it does not exist.
# The HCS application does not reside here, but we're going to keep the version information here.
if [[ ! -d /var/www/html/hcs ]]; then
  mkdir /var/www/html/hcs
fi

################### root cron ##########################################################################################
################ 2019 08 09 Changes in apache cron to support jobs to be run only once if it is part of cluster ########
#verifying apache jobs, and migrating them to akkadianuser
root_cron_content=$(sudo crontab -u root -l)
if [[ "$root_cron_content" != *"write_master.sh"* ]]; then

  root_new_cron_content="*/1 * * * * sh /home/akkadianuser/scripts/ha/write_master.sh"
  (
    sudo crontab -u root -l
    echo "$root_new_cron_content"
  ) | sudo crontab -u root -
fi

if [[ "$root_cron_content" == *"read_and_eval.sh"* ]]; then
  sudo crontab -u root -l | grep -v 'read_and_eval.sh' | crontab -u root -
fi

if [[ "$root_cron_content" != *"PATH="* ]]; then
  root_new_cron_content="PATH=\"/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin\""
  (
    sudo crontab -u root -l
    echo "$root_new_cron_content"
  ) | sudo crontab -u root -
fi

if [[ -d /var/www/html/asb ]]; then
  if [[ "$root_cron_content" != *"@reboot php /var/www/biosuuid.php"* ]]; then
    root_new_cron_content=$"@reboot php /var/www/biosuuid.php"
    (
      sudo crontab -u root -l
      echo "$root_new_cron_content"
    ) | sudo crontab -u root -
  else
    if [[ "$root_cron_content" == *"@reboot /home/akkadianuser/scripts/apm_cm_runner.sh \"php /var/www/biosuuid.php/\""* ]]; then
      root="@reboot /home/akkadianuser/scripts/apm_cm_runner.sh \"php /var/www/biosuuid.php/\""
      (
        sudo crontab -u root -l
        echo "$root"
      ) | sudo crontab -u root -
    fi
  fi

fi
########################################################################################################################

################# akkadianuser cron ####################################################################################
akkadian_cron_content=$(sudo crontab -u akkadianuser -l)
if [[ "$akkadian_cron_content" != *"read_and_eval.sh"* ]]; then
  akkadian="*/1 * * * * sh /var/www/product_update/read_and_eval.sh > /var/www/product_update/crontab.log 2>&1"
  (
    sudo crontab -u akkadianuser -l
    echo "$akkadian"
  ) | sudo crontab -u akkadianuser -
fi

##### Removing unnecessary entries ###################
if [[ "$akkadian_cron_content" == *"@reboot python2.7 /home/akkadianuser/executor.py"* ]]; then
  sudo crontab -u akkadianuser -l | grep -v 'python2.7 /home/akkadianuser/executor.py' | crontab -u akkadianuser -
fi
########################################################################################################################

################## apache cron #########################################################################################
#### Checking on old versions to add or not the apache cron
#### DO  NOT MODIFY TABS IN apache_cron_content!!!!!!!!!!!!!!
if [[ ! -d /opt/akkadian/aco-server ]]; then

  apache_original=$(sudo crontab -u apache -l)
  if [[ -d /var/www/html/pme ]]; then

    if [[ "$apache_original" != *"php /var/www/html/pme/index.php --task=Sync"* ]]; then
      apache=$"5 * * * * sh /home/akkadianuser/scripts/apm_cm_runner.sh \"php /var/www/html/pme/index.php --task=Sync\" "
      (
        sudo crontab -u apache -l
        echo "$apache"
      ) | sudo crontab -u apache -
    fi

    if [[ "$apache_original" != *"php /var/www/html/pme/index.php --task=Schedule"* ]]; then
      apache=$"5 * * * * sh /home/akkadianuser/scripts/apm_cm_runner.sh \"php /var/www/html/pme/index.php --task=Schedule\" "
      (
        sudo crontab -u apache -l
        echo "$apache"
      ) | sudo crontab -u apache -
    fi

    if [[ "$apache_original" != *"php /var/www/html/pme/index.php --task=Schedule"* ]]; then
      apache=$"5 * * * * sh /home/akkadianuser/scripts/apm_cm_runner.sh \"php /var/www/html/pme/index.php --task=BackupSchedule\" "
      (
        sudo crontab -u apache -l
        echo "$apache"
      ) | sudo crontab -u apache -
    fi

    if [[ -f /home/akkadianuser/configs/akkadian_apm.cnf ]]; then
      sudo cp -Rf /home/akkadianuser/configs/akkadian_apm.cnf /etc/my.cnf.d/akkadian.cnf
    fi

  fi

  if [[ -d /var/www/html/asb ]]; then

    if [[ "$apache_original" != *"php /var/www/html/asb/artisan schedule:run"* ]]; then
      apache=$"* * * * * sh /home/akkadianuser/scripts/apm_cm_runner.sh \"php /var/www/html/asb/artisan schedule:run\" "
      (
        sudo crontab -u apache -l
        echo "$apache"
      ) | sudo crontab -u apache -
    fi

    if [[ "$apache_original" == *"@reboot php /var/www/biosuuid.php"* ]]; then
      sudo crontab -u apache -l | grep -v '@reboot php /var/www/biosuuid.php' | crontab -u apache -
    fi
  fi

  if [[ -d /var/www/html/orc ]]; then

    if [[ "$apache_original" != *"php /var/www/html/orc/artisan schedule:run"* ]]; then
      apache=$"* * * * * sh /home/akkadianuser/scripts/apm_cm_runner.sh \"php /var/www/html/orc/artisan schedule:run\" "
      (
        sudo crontab -u apache -l
        echo "$apache"
      ) | sudo crontab -u apache -
    fi
  fi

  if [[ -d /var/www/html/acm ]]; then

    if [[ "$apache_original" != *"/usr/bin/php /var/www/html/acm/scheduler.php"* ]]; then
      apache_cron_content=$"* * * * * sh /home/akkadianuser/scripts/apm_cm_runner.sh \"/usr/bin/php /var/www/html/acm/scheduler.php > /dev/null 2>&1\" "
      (
        sudo crontab -u apache -l
        echo "$apache"
      ) | sudo crontab -u apache -
    fi
    if [[ -f /home/akkadianuser/configs/akkadian_acm.cnf ]]; then
      sudo cp -Rf /home/akkadianuser/configs/akkadian_acm.cnf /etc/my.cnf.d/akkadian.cnf
    fi
  fi
else # Here all code tied to aco!!!!!

  echo 'aCO requires noode, gcc and c++ installing them...' >>/home/akkadianuser/logs/cli_installation.log

  isNodeInstalled=$(node -v)
  isSNMPInstalled=$(sudo systemctl status snmpd)

  if [[ "$isNodeInstalled" == "" ]]; then
    #If it is aco, then install nodejs
    sudo yum -y install gcc-c++ make
    sudo curl -sL https://rpm.nodesource.com/setup_6.x | sudo -E bash -
    sudo yum -y install nodejs
  fi

  if grep -q "Active: active (running)" <<<"$isSNMPInstalled"; then
    echo "Nothing to do, snmpd is already installed"
  else
    sudo yum -y install net-snmp net-snmp-utils
    sudo systemctl enable snmpd
    sudo systemctl start snmpd
  fi

fi

if [[ ! -d /opt/akkadian/aco-server ]]; then
  sudo chown -Rf apache /etc/openldap/certs/
  sudo chown -Rf apache /etc/pki/ca-trust/extracted/pem/
  sudo chown -Rf apache /etc/pki/tls/certs/
  sudo chown -Rf apache /etc/pki/ca-trust/source/anchors/
fi

#sudo rm -Rf /var/www/cli_updates/*

#### Installing executor.py ########
executor_service_file=/etc/systemd/system/akkadian-executor.service
if [[ ! -f ${executor_service_file} ]]; then

  #This is because new openssl version
  sudo echo "/usr/local/lib64" >/etc/ld.so.conf.d/openssl.conf
  sudo ldconfig

fi

# If akkadian-executor.service does not exist or exists and has a size equal to zero.
# The size equal to zero check is being added because we noticed in the ACM server that
# akkadian-executor.service is present but its size is equal to zero. This check should
# fix broken OVAs that have been distributed to customers.
if [[ ! -s ${executor_service_file} ]]; then

  sudo cp /home/akkadianuser/scripts/executor/akkadian-executor.service /etc/systemd/system/

  sudo systemctl daemon-reload
  sudo systemctl enable akkadian-executor
  sudo systemctl start akkadian-executor

fi

if [[ ! -f /home/backups ]]; then
  sudo mkdir /home/backups
fi

if [[ -d /var/log ]]; then
  sudo chmod -Rf 755 /var/log
fi

if [[ -d /var/lib/mysql/pme ]]; then
  sudo chmod -Rf 755 /var/lib/mysql/pme
fi

if [[ -d /home/akkadianuser/aUU_installer ]]; then

  if [[ ! -d /var/www/html/update ]]; then
    sudo mkdir /var/www/html/update
  fi

  aUUName=$(basename $(ls /home/akkadianuser/aUU_installer/*.tar.gz | tail -1))

  if [[ ! -d /tmp/installer/ ]]; then
    sudo mkdir /tmp/installer/
  fi

  #sudo yum -y install git
  cp -Rf /home/akkadianuser/aUU_installer/$aUUName /tmp/installer/

  # Destination folder is different in case aco or pme
  if [[ -d /opt/akkadian/aco-server ]]; then # if aco then use nginx
    #install php to support auu, this need to be done before getting nginx restarted.
    internet=$(sh /home/akkadianuser/scripts/check_internet.sh)
    if [[ "$internet" == "True" ]]; then
      version=$(php --version)
      regex="^PHP\s(\d\.\d\.\d*)\s"
      $is_php_installed=$(echo "False")
      if [[ $version =~ regex ]]; then
        if [[ $1 > "7.4" ]]; then
          echo "------------ PHP is already installed ------------"
          is_php_installed=$(echo "True")
        fi
      fi

      if [[ "$is_php_installed" == "True" ]]; then
        #if php is installed mean the folder update is there
        sudo sh /home/akkadianuser/scripts/universal_updater/universal_updator_installer.sh /tmp/installer/$aUUName /usr/share/nginx/html
      else
        mkdir /usr/share/nginx/html/update
        git init /usr/share/nginx/html/update
        sh /home/akkadianuser/upgrades/php_into_acs/php-installation-script.sh
        sudo sh /home/akkadianuser/scripts/universal_updater/universal_updator_installer.sh /tmp/installer/$aUUName /usr/share/nginx/html/update
        sudo chown nginx:nginx /var/www/product_update
        sleep 3
        sudo systemctl restart nginx
      fi
    else
      echo "Server without Internet access, installing auu"
      if [[ "$is_php_installed" == "True" ]]; then
        #if php is installed mean the folder update is there
        sudo sh /home/akkadianuser/scripts/universal_updater/universal_updator_installer.sh /tmp/installer/$aUUName /usr/share/nginx/html
      fi
    fi
  else #if pme then use apache
    sudo sh /home/akkadianuser/scripts/universal_updater/universal_updator_installer.sh /tmp/installer/$aUUName /usr/share/nginx/html/update
  fi

  sudo rm -Rf /home/akkadianuser/aUU_installer

  sudo systemctl stop akkadian-executor
  sleep 3
  sudo systemctl start akkadian-executor
fi

if [[ -f /home/akkadianuser/global_variables/restart_executor ]]; then
  needed_restart=$(cat /home/akkadianuser/global_variables/restart_executor)
  if [[ "$needed_restart" == "yes" ]]; then
    sudo systemctl restart akkadian-executor
    sudo echo "no" >/home/akkadianuser/global_variables/restart_executor
  fi
fi

if [[ -d /opt/akkadian/aco-server ]]; then

  FILE_SSHD="/home/akkadianuser/scripts/security/patch_sshd_1.sh"
  FILE_GRUB2="/home/akkadianuser/scripts/security/patch_grub2_1.sh"
  FILE_ENGINEX="/home/akkadianuser/scripts/security/patch_nginex_1.sh"
  # FILE_PWD="/home/akkadianuser/scripts/security/patch_pwd_1.sh"

  if [[ -f $FILE_SSHD ]]; then
    sh $FILE_SSHD
  fi

  if [[ -f $FILE_GRUB2 ]]; then
    sh $FILE_GRUB2
  fi

  if [[ -f $FILE_ENGINEX ]]; then
    sh $FILE_ENGINEX
  fi

  #if [[ -f $FILE_PWD ]]; then
  #    sh $FILE_PWD
  #fi
fi

################################ Preventing ctrl-c ctrl-z   ######################################################

if grep -Fxq "trap \"\" 2" /home/akkadianuser/.bashrc; then
  echo "nothing"
else
  sed -i '1s/^/trap "" 2\n/' /home/akkadianuser/.bashrc
  sed -i '1s/^/trap "" 20\n/' /home/akkadianuser/.bashrc
fi

# Preventing things like: ssh username@ip something, so after running akk app manager, exit.
if grep -Fxq "exit" /home/akkadianuser/.bashrc; then
  echo "nothing"
else
  printf "\nexit" >>/home/akkadianuser/.bashrc
fi

################# Preventing things like: ssh username@ip something, so after running akk app manager, exit.

############## UPGRADES SECTION!!!! ##################################
#### Make the script auto checker, so the idea is to check inside the script if we need to run it or not!

sudo sh /home/akkadianuser/upgrades/akkadian_repo/add_akkadian_repo.sh
sudo sh /home/akkadianuser/upgrades/python3.7/install_python379.sh

############# This is needed to avoid mariadb bin logs files take too much #################

akkadianuser_cron=$(crontab -u akkadianuser -l)
if [[ "$akkadianuser_cron" != *"manage_db_logs.sh"* ]]; then
  akkadianuser_new_cron_content="0 * * * * sh /home/akkadianuser/scripts/manage_db_logs.sh"
  (
    sudo crontab -u akkadianuser -l
    echo "$akkadianuser_new_cron_content"
  ) | sudo crontab -u akkadianuser -

fi

###########################################################################################

#TODO: uncomment these lines when the cli 4.0 is ready to support .net core 5.0
#version=$(dotnet --version)
#if [[ "$version" != "5.0."* ]]; then
#  sudo rpm -Uvh https://packages.microsoft.com/config/centos/7/packages-microsoft-prod.rpm
#  sudo yum -y install dotnet-sdk-5.0
#fi
########################################################################
