#!/usr/bin/env sh

base=/home/akkadianuser/upgrades
scripts=/home/akkadianuser/scripts
packages=/home/akkadianuser/packages

jdkPackagePath=/opt/jdk-17.0.1

#check to see if ucce service is running
#ucceServiceStatus=$(sh $scripts/ucce/statusUcce.sh | grep 'active' | awk '{print $2}')
#remove leading and trailing spaces from package
zipUCCEPackage=$(find "$packages/ucce/ucce-api"  -maxdepth 1 -name '*.zip' | sed -e 's/^[[:space:]]*//')
echo "Finding package: $zipUCCEPackage" >> /var/www/product_update/update.log

#get the package name
uccePackage=$(basename "$zipUCCEPackage")
echo "Finding basename to package: $uccePackage" >> /var/www/product_update/update.log

# this will give me the version of the zip file
newUcceAPIVersion=$(echo "$uccePackage" | sed 's/(ucce-api-|ucce-service-)//' | sed 's/.zip//')
ucceBackup="/home/backups/ucce-service"
ucceServicePath="/opt/ucce-service"

openJDKTar=$(find "$packages" -maxdepth 1 -name "openjdk*.tar.gz" | sed -e 's/^[[:space:]]*//')
openJDKPackage=$(basename "$openJDKTar")
openJDKVersion=$(echo "$openJDKPackage" | sed 's/openjdk-//' | sed 's/_linux-x64_bin.tar.gz//')

if [ ! -d "$ucceBackup" ]; then
  mkdir -p "$ucceBackup"
fi

if [ ! -d /opt/jdk ]; then
  mkdir -p /opt/jdk
fi

if [ ! -d "$ucceServicePath" ]; then
  echo "Create ucce-service directory" >> /var/www/product_update/update.log
  #will create a new ucce service for this new version
  echo "Creating UCCE Service Path $ucceServicePath." >> /var/www/product_update/update.log
  mkdir -p "$ucceServicePath"
fi

if [ ! -f /opt/ucce-service/version ]; then
  echo "Creating ucce version file" >> /var/www/product_update/update.log
  echo "$newUcceAPIVersion" > /opt/ucce-service/version
fi

if [ ! -f /opt/jdk/version ]; then
  echo "Creating openjdk version file" >> /var/www/product_update/update.log
  echo "$openJDKVersion" > /opt/jdk/version
fi


#instead of doing this we can probably check the version that is installed in case we need to update java in the future
#command for that is: /opt/jdk-17.0.1/bin/java --version | head -1 | awk '{print $2}' but we may need to place this in the $PATH var
if [ ! -d $jdkPackagePath ]; then
  echo "Unpacking JDK into /opt" >> /var/www/product_update/update.log
  tar -zxvf "$packages/$openJDKPackage" -C /opt/
  rm -f "$packages/$openJDKPackage"
fi

ucceAPIInSystem=$(cat /opt/ucce-service/version)
restartUcce=0
#if it's a new zip meaning new version then unzip it
echo "Will validate version check" >> /var/www/product_update/update.log
echo "Old UCCE API Version: $ucceAPIInSystem"  >> /var/www/product_update/update.log
echo "New UCCE API Version: $newUcceAPIVersion"  >> /var/www/product_update/update.log
if [ "$ucceAPIInSystem" != "$newUcceAPIVersion" ];then
  echo "Validated version check and writing new version. Will restart UCCE Service" >> /var/www/product_update/update.log
  restartUcce=1
  echo "$newUcceAPIVersion" > /opt/ucce-service/version
  #stopping ucce service so that new packing unzips without a problem because it was failing when i just restarted
  sh $scripts/ucce/stopUcce.sh
  echo "Unzipping UCCE Service and overwriting for potential updates to service" >> /var/www/product_update/update.log
  unzip -o "$packages/ucce/ucce-api/$uccePackage" -d $ucceServicePath
  rm -f "$packages/ucce/ucce-api/$uccePackage"
fi

openJDKInSystem=$(cat /opt/jdk/version)
if [ "$openJDKVersion" != "$openJDKInSystem" ]; then
  echo "Unpacking New Open JDK Version $openJDKVersion  into /opt" >> /var/www/product_update/update.log
  tar -zxvf "$packages/$openJDKPackage" -C /opt/
  echo "Removing OpenJDK Tarball from system"  >> /var/www/product_update/update.log
  rm -f "$packages/$openJDKPackage"
fi

#Don't think I need this condition as the file doesn't exist.
#if [ ! -f  "$ucceServicePath/ucce-api-1.0.1.jar" ]; then
#  echo "Unzipping UCCE Service and overwriting for potential updates to service" >> /var/www/product_update/update.log
#  unzip -o "$scripts/ucce/ucce-api/$uccePackage" -d $ucceServicePath
#  rm -f "$scripts/ucce/ucce-api/$uccePackage"
#fi

ucceServiceFile="/etc/systemd/system/ucce.service"

# shellcheck disable=SC2039
if [ -s $ucceServiceFile ]; then
  declarations=$(grep -c "$ucceServicePath/ucce-api-1.0.1.jar" $ucceServiceFile)
  if ((declarations == 1));  then
    echo "Rewriting UCCE service to make jar file a generic name."  >> /var/www/product_update/update.log
    sed -i 's/\/opt\/ucce-service\/ucce-api-1.0.1.jar/\/opt\/ucce-service\/ucce-api.jar/' $ucceServiceFile
  fi
fi

#ExecStart=/usr/bin/java -jar /opt/ucce-service/ucce-api-1.0.jar
if [ ! -s /etc/systemd/system/ucce.service ]; then
  echo "Copying UCCE Service into systemd" >> /var/www/product_update/update.log
  cp "${scripts}/ucce/ucce.service" /etc/systemd/system
  echo "Reloading daemon" >> /var/www/product_update/update.log
  systemctl daemon-reload
  echo "Starting UCCE service" >> /var/www/product_update/update.log
  sh $scripts/ucce/startUcce.sh
fi

ucceIsEnabled=$(systemctl list-unit-files | grep ucce.service  | awk '{print $2}')
if [ "$ucceIsEnabled" != "enabled" ]; then
  echo "Enabling UCCE service" >> /var/www/product_update/update.log
  sh $scripts/ucce/enableUcce.sh
fi

if [ $restartUcce == 1 ];then
  echo "Starting UCCE Service due installing new package ucce api package." >> /var/www/product_update/update.log
   #starting ucce service so that new packing takes affect
 sh $scripts/ucce/startUcce.sh
fi