import click
import subprocess
from src.security_handler.security_handler import is_ssh_enable, disable_ssh, enable_ssh, is_allow_root_access_on, \
    disable_root_ssh_access, enable_root_ssh_access, port_enabled, open_port, close_port
import configparser

mapper = configparser.RawConfigParser()
mapper.read('mapper.ini')


def render_security_menu():
    click.clear()
    click.secho('Akkadian Security Management', fg='green')

    is_ssh = is_ssh_enable()
    is_allow_root_access = is_allow_root_access_on()
    is_port_80_open = port_enabled('80')

    echo_current_security_configuration_section(is_ssh, is_allow_root_access, is_port_80_open)
    echo_option_section(is_ssh, is_allow_root_access, is_port_80_open)
    click.secho('b: Back to Main Menu', fg='red')


def echo_current_security_configuration_section(is_ssh, is_allow_root_access, is_port_80_open):
    click.secho('Current Security Configuration', fg='yellow')
    if is_ssh:
        click.secho("SSH access is enabled", fg='green')
    else:
        click.secho("SSH access is disabled", fg='green')

    if is_allow_root_access:
        click.secho("Root user SSH access is enabled", fg='green')
    else:
        click.secho("Root user SSH access is disabled", fg='green')

    if is_port_80_open:
        click.secho("Port 80 is open", fg='green')
    else:
        click.secho("Port 80 is closed", fg='green')


def echo_option_section(is_ssh, is_allow_root_access, is_port_80_open):
    click.secho('Security Options', fg='yellow')
    if is_ssh:
        click.secho('1: Disable SSH access', fg='green')
    else:
        click.secho('1: Enable SSH access', fg='green')

    if is_allow_root_access:
        click.secho('2: Disable root SSH access', fg='green')
    else:
        click.secho('2: Enable root SSH access', fg='green')

    if is_port_80_open:
        click.secho('3: Close port 80', fg='green')
    else:
        click.secho('3: Open port 80', fg='green')

    click.secho("4: Change akkadianuser's password", fg='green')


def security_menu_handler(char, menu):
    is_ssh = is_ssh_enable()
    is_allow_root_access = is_allow_root_access_on()
    is_port_80_open = port_enabled('80')

    if char == 'b':
        menu = 'main'
    elif char == '1' and is_ssh:
        disable_ssh()
    elif char == '1' and not is_ssh:
        enable_ssh()
    elif char == '2' and is_allow_root_access:
        disable_root_ssh_access()
    elif char == '2' and not is_allow_root_access:
        enable_root_ssh_access()
    elif char == '3' and is_port_80_open:
        close_port('80')
    elif char == '3' and not is_port_80_open:
        open_port('80')
    elif char == '4':
        click.secho('Hit enter twice without input to cancel the change password action.', fg='yellow')
        subprocess.call(['passwd', 'akkadianuser'])

    return menu
