import time
from subprocess import Popen, PIPE

from src.ha_handler.ha_handler import is_full_ha_mode
from src.cli_commons.commons_manager import (installed_application, show_select_option)

import click

import configparser
mapper = configparser.RawConfigParser()
mapper.read('mapper.ini')

logs_path = configparser.RawConfigParser()
logs_path.read('logs.ini')


def supervisor_stop():
    click.echo('Stopping...')
    p = Popen(["sh", mapper.get('GeneralScripts', 'supervisorStop')], stdout=PIPE, stderr=PIPE)
    for line in p.stdout:
        line = line.decode("utf-8")
        print(line)
    p.wait()

    if p.returncode != 0:
        click.secho('[FAIL] Supervisor Service could not be stopped. Is it running?', fg="red")
    else:
        click.secho('[OK] Supervisor Service is down.', fg="yellow")

    time.sleep(3)


def supervisor_start():
    click.echo('Starting...')
    p = Popen(["sh", mapper.get('GeneralScripts', 'supervisorStart')], stdout=PIPE, stderr=PIPE)
    for line in p.stdout:
        line = line.decode("utf-8")
        print(line)
    p.wait()

    if p.returncode != 0:
        click.secho('[FAIL] Supervisor Service Could Not Be Started', fg="red")
    else:
        click.secho('[OK] Supervisor Started Successfully', fg="yellow")

    time.sleep(3)


def supervisor_restart():
    click.echo('Restarting...')
    p = Popen(["sh", mapper.get('GeneralScripts', 'supervisorRestart')], stdout=PIPE, stderr=PIPE)
    for line in p.stdout:
        line = line.decode("utf-8")
        print(line)
    p.wait()

    if p.returncode != 0:
        click.secho('[FAIL] Supervisor Could Not Be Restarted', fg="red")
    else:
        click.secho('[OK] Supervisor Service Restarted successfully', fg="yellow")

    time.sleep(3)


def supervisor_status():
    p = Popen(["sh", mapper.get('GeneralScripts', 'supervisorStatus')], stdout=PIPE, stderr=PIPE)
    for line in p.stdout:
        line = line.decode("utf-8")
        print(line)
    p.wait()

    click.pause('Please, press any key to continue...')

def supervisor_workers():
    p = Popen(["sh", mapper.get('GeneralScripts', 'supervisorWorkers')], stdout=PIPE, stderr=PIPE)
    for line in p.stdout:
        line = line.decode("utf-8")
        print(line)
    p.wait()

    click.pause('Please, press any key to continue...')

def supervisor_logs():
    """
    only showing supervisord.log.
    other supervisord.log files are dated. I don't really see the value in allowing the user to view those. But also it's actually really complex because
    I would not know what dates exist.
    """
    option = supervisor_logs_menu_manager()
    supervisor_logs_menu_handler(option)

def supervisor_logs_menu_handler(option):
    if option == '1':
        p = Popen("nano -v /var/log/supervisor/supervisord.log", shell=True)
        stdout = p.communicate()
        print(stdout)

    if option == 'b':
        return
    else:
        supervisor_logs()

def supervisor_logs_menu_manager():
    p = Popen(["sh", mapper.get('GeneralScripts', 'logsSupervisor')], stdout=PIPE, stderr=PIPE)
    click.clear()
    lines = []
    i = 1
    supervisor = "supervisord.log"
    click.secho('Supervisor Logs', fg='green')
    for line in p.stdout:
        line = line.decode("utf-8")
        if supervisor == line.strip():
            lines.append('%s: ' % str(i) + ' ' + line.strip())
            i += 1
    for line in lines:
        click.secho(line, fg="green")

    click.secho('b: Back to Supervisor Service Settings', fg='red')
    click.secho('')
    click.secho('Once option is selected you can press \'q\' at any time\nto return to back to this menu.', fg='yellow')
    click.secho('\nSelect an option: ', fg='white', blink=True, nl=False)
    char = click.getchar()
    click.echo('')
    return char

def supervisor_worker_logs():
    installedApp = installed_application()
    ha_enabled = is_full_ha_mode(installedApp)
    option = supervisor_worker_logs_menu_manager(ha_enabled)
    supervisor_worker_logs_menu_handler(option, ha_enabled)

def supervisor_worker_logs_menu_handler(option, ha_enabled):
    if option == '1':
        p = Popen("nano -v /var/log/supervisor/apm_bulk_worker_manager.0.stdout.log", shell=True)
        stdout = p.communicate()
        print(stdout)
    if option == '2':
        p = Popen("nano -v /var/log/supervisor/apm_bulk_worker_manager.1.stdout.log", shell=True)
        stdout = p.communicate()
        print(stdout)
    if option == '3':
        if ha_enabled == 'True':
            p = Popen("nano -v /var/log/supervisor/apm_bulk_worker_manager.2.stdout.log", shell=True)
        else:
            p = Popen("nano -v /var/log/supervisor/apm_realtime_worker_manager.0.stdout.log", shell=True)
        stdout = p.communicate()
        print(stdout)
    if option == '4':
        if ha_enabled == 'True':
            p = Popen("nano -v /var/log/supervisor/apm_realtime_worker_manager.0.stdout.log", shell=True)
        else:
            p = Popen("nano -v /var/log/supervisor/apm_realtime_worker_manager.1.stdout.log", shell=True)
        stdout = p.communicate()
        print(stdout)
    if option == '5':
        if ha_enabled == 'True':
            p = Popen("nano -v /var/log/supervisor/apm_realtime_worker_manager.1.stdout.log", shell=True)
        else:
            p = Popen("nano -v /var/log/supervisor/apm_realtime_worker_manager.2.stdout.log", shell=True)
        stdout = p.communicate()
        print(stdout)
    if option == '6':
        if ha_enabled == 'True':
            p = Popen("nano -v /var/log/supervisor/apm_realtime_worker_manager.2.stdout.log", shell=True)
        else:
            p = Popen("nano -v /var/log/supervisor/apm_realtime_worker_manager.3.stdout.log", shell=True)
        stdout = p.communicate()
        print(stdout)
    if option == '7':
        if ha_enabled == 'True':
            p = Popen("nano -v /var/log/supervisor/apm_realtime_worker_manager.3.stdout.log", shell=True)
        else:
            p = Popen("nano -v /var/log/supervisor/apm_realtime_worker_manager.4.stdout.log", shell=True)
        stdout = p.communicate()
        print(stdout)
    if option == 'b':
        return
    else:
        supervisor_worker_logs()

def supervisor_worker_logs_menu_manager(ha_enabled):
    p = Popen(["sh", mapper.get('GeneralScripts', 'logsSupervisor')], stdout=PIPE, stderr=PIPE)
    click.clear()
    lines = []
    i = 1
    supervisor = "supervisord"
    click.secho('Supervisor Worker Logs', fg='green')
    for line in p.stdout:
        line = line.decode("utf-8")
        if supervisor not in line:
            if ha_enabled is True:
                if i < 7:
                    lines.append('%s: ' % str(i) + ' ' + line.strip())
            else:
                lines.append('%s: ' % str(i) + ' ' + line.strip())
            i += 1
    for line in lines:
        click.secho(line, fg="green")

    click.secho('b: Back to Supervisor Service Settings', fg='red')
    click.secho('')
    click.secho('Once option is selected you can press \'q\' at any time\nto return to back to this menu.', fg='yellow')
    click.secho('\nSelect an option: ', fg='white', blink=True, nl=False)
    char = click.getchar()
    click.echo('')
    return char


def read_log(log_path):
    log_file_path = logs_path.get('Supervisord', log_path)

    try:
        with open(log_file_path, 'r') as f:
            for line in f:
                line = line.strip().rstrip()
                click.secho(line, fg='green')
    except Exception as e:
        click.secho('The file does not exist', fg='green')

    click.pause('Please press any key to continue...')