import time
from subprocess import Popen, PIPE

import click

import configparser
mapper = configparser.RawConfigParser()
mapper.read('mapper.ini')


def ucce_install():
    click.echo('Installing/Updating UCCE Services')
    p = Popen(["sh", mapper.get('GeneralScripts', 'ucceInstall')], stdout=PIPE, stderr=PIPE)
    for line in p.stdout:
        line = line.decode("utf-8")
        print(line)
    p.wait()
    out, err = p.communicate()
    time.sleep(10)

    if p.returncode != 0:
        click.secho("Warning/Error", fg="red")
        click.secho("Output:", fg="red")
        click.secho(out, fg="red")
        click.secho("Error:", fg="red")
        click.secho(err, fg="red")
        return
    else:
        click.pause('UCCE Services were properly installed or updated. Press any key to continue...')
        return

def ucce_stop():
    click.echo('Stopping...')
    p = Popen(["sh", mapper.get('GeneralScripts', 'ucceStop')], stdout=PIPE, stderr=PIPE)
    for line in p.stdout:
        line = line.decode("utf-8")
        print(line)
    p.wait()

    if p.returncode != 0:
        click.secho('[FAIL] UCCE Service could not be stopped. Is it running?', fg="red")
    else:
        click.secho('[OK] UCCE Service is down.', fg="yellow")

    time.sleep(3)


def ucce_start():
    click.echo('Starting...')
    p = Popen(["sh", mapper.get('GeneralScripts', 'ucceStart')], stdout=PIPE, stderr=PIPE)
    for line in p.stdout:
        line = line.decode("utf-8")
        print(line)
    p.wait()

    if p.returncode != 0:
        click.secho('[FAIL] UCCE Service Could Not Be Started', fg="red")
    else:
        click.secho('[OK] UCCE Started Successfully', fg="yellow")

    time.sleep(3)


def ucce_restart():
    click.echo('Restarting...')
    p = Popen(["sh", mapper.get('GeneralScripts', 'ucceRestart')], stdout=PIPE, stderr=PIPE)
    for line in p.stdout:
        line = line.decode("utf-8")
        print(line)
    p.wait()

    if p.returncode != 0:
        click.secho('[FAIL] UCCE Could Not Be Restarted', fg="red")
    else:
        click.secho('[OK] UCCE Service Restarted successfully', fg="yellow")

    time.sleep(3)


def ucce_status():
    p = Popen(["sh", mapper.get('GeneralScripts', 'ucceStatus')], stdout=PIPE, stderr=PIPE)
    for line in p.stdout:
        line = line.decode("utf-8")
        print(line)
    p.wait()

    click.pause('Please, press any key to continue...')

def ucce_rest_api_log():
    p = Popen("nano -v /opt/ucce-service/ucce-api.log", shell=True)
    stdout = p.communicate()
    print(stdout)

def ucce_conn_api_log():
    p = Popen("nano -v /opt/ucce-service/CMSLOG", shell=True)
    stdout = p.communicate()
    print(stdout)