# Add here the steps needed common to all products
base=$(echo "/var/www/product_update/scripts/")

echo "Checking if either HA or replication is enabled before proceeding." >>/var/www/product_update/update.log
if sh "${base}ha_is_off_or_in_maintenance_mode.sh"; then
    # HA is off or maintenance mode is on, so it is safe to block port 3306.
    # Enabling HA or database replication or disabling maintenance mode will unblock port 3306.
    sudo firewall-cmd --remove-port=3306/tcp --permanent
    sudo firewall-cmd --reload
else
    # HA is on or replication is on and maintenance mode is off: block the install.
    exit 1
fi

# Here we will check which product is installed and compare it to the product the update is intended for.
# If the products do not match and the product is not tui, then exit this script.

if ! bash "${base}update_matches_product_check.sh"; then
    exit 1
fi

#This solves the problem with the non-Internet connection clients - If it is using auu means jq is already in the machine,
#so remove the line in installer and in after_installer scripts.
sudo sed -i '/sudo yum -y install jq/d' /home/akkadianuser/scripts/auto_update.sh
#yes | sudo cp tmp  /home/akkadianuser/scripts/auto_update.sh

sudo sed -i '/sudo yum -y install jq/d' /home/akkadianuser/scripts/after_installation/after_installation_needed_scripts.sh
#yes | sudo cp tmp2  /home/akkadianuser/scripts/after_installation/after_installation_needed_scripts.sh
#rm -Rf tmp tmp2
#######################################################################


if [[ -s /home/akkadianuser/upgrades/auto_updates_updates/auto_update.sh ]]; then
    \cp -Rf /home/akkadianuser/upgrades/auto_updates_updates/auto_update.sh /home/akkadianuser/scripts/auto_update.sh
fi

#Ensuring the new version of the auto_update.sh is the one we will run and
#run it from a different location since it will affect the auto install.
if [[ -d /tmp/cli_installer ]]; then
  rm -Rf /tmp/cli_installer
fi

sudo mkdir /tmp/cli_installer
cli_name=$(ls /var/www/product_update/updates/tui*.tar.gz)
sudo tar -xvzf ${cli_name} -C /tmp/cli_installer
cp -Rf /tmp/cli_installer/scripts/universal_updater/update_cli.sh /home/akkadianuser/scripts/universal_updater/update_cli.sh
cp -Rf /tmp/cli_installer/scripts/auto_update.sh /var/www/product_update/

#############################################################################


if [[ -d /var/www/html/acm/ ]]; then
    sudo bash "${base}acm_openssl_version_fix.sh"
fi

######### Running security migrations #################################
if [[ ! -d /opt/akkadian/aco-server/ ]]; then
  if ! sh "${base}pre_install_validation_checks.sh"; then
    echo "Some installation validations were not accomplished" >>/var/www/product_update/update.log
    exit 1
  fi

  echo "It is not aco, updating db" >>/var/www/product_update/update.log
  sh "${base}update_db.sh"
fi

# sh $base"close_ssh_port.sh"
sh "${base}update_support_user.sh"