are_pcsd_pacemaker_and_corosync_running() {
    result="False"
    pcsd_status=$(systemctl is-active --quiet pcsd && echo Service is running)
    pacemaker_status=$(systemctl is-active --quiet pacemaker && echo Service is running)
    corosync_status=$(systemctl is-active --quiet corosync && echo Service is running)

    if [[ "$pcsd_status" == "Service is running" && "$pacemaker_status" == "Service is running" && "$corosync_status" == "Service is running" ]]; then
        result="True"
    fi

    echo "$result"
}

check_galera_replication_status() {
    status=$(mysql --defaults-extra-file=/home/akkadianuser/.my.cnf -u root -e "SHOW GLOBAL STATUS LIKE 'wsrep_local_state_comment'" | awk '{print $2}' | tail -1)
    echo "$status"
}

is_redundancy_enable_galera() {
    result="False"
    isRunning=$(check_galera_replication_status)
    if [[ -n "$isRunning" ]]; then
        result="True"
    fi
    echo "$result"
}

is_mongo_replication_set_in_config_file() {
    # We cannot ensure mongo is up testing the configuration at mongo level. Maybe in this particular moment mongo is down
    nodesConfigured="False"

    if [[ -f /home/akkadianuser/load_balancer.config ]]; then
        nodes=$(head -n 1 /home/akkadianuser/load_balancer.config)
        if [[ ! -v "${nodes}" ]]; then
            let COUNTER=COUNTER+1
            nodesConfigured="True"
        fi
    fi

    echo "$nodesConfigured"

}

maintenance_is_mode_on() {
    if [[ -s /home/akkadianuser/maintenance_info.config ]]; then
        echo "True"
    else
        echo "False"
    fi
}

installed_app() {
    if [[ -d /opt/akkadian/aco-server ]]; then
        echo "aco"
    elif [[ (-d /var/www/html/pme) && (-d /var/www/html/orc) && (-d /var/www/html/asb) ]]; then
        echo "hcs"
    elif [[ -d /var/www/html/pme ]]; then
        echo "pme"
    elif [[ -d /var/www/html/acm ]]; then
        echo "acm"
    elif [[ -d /var/www/html/asb ]]; then
        echo "asb"
    elif [[ -f /usr/local/bin/AkkadianCloudConnector ]]; then
        echo "acc"
    fi
}

product=$(installed_app)

echo "Checking if HA is enabled." >>/var/www/product_update/update.log
# returns "True" or "False"
pacemaker_and_corosync_are_running=$(are_pcsd_pacemaker_and_corosync_running)

echo "Checking if replication is enabled." >>/var/www/product_update/update.log
if [[ $product == "pme" ]]; then
    # Returns void or "True"
    replication_is_enabled=$(is_redundancy_enable_galera)
elif [[ $product == "aco" ]]; then
    # Returns "True" or "False"
    replication_is_enabled=$(is_mongo_replication_set_in_config_file)
else
    # Returns void or "True"
    replication_is_enabled=$(is_redundancy_enable_galera)
fi

echo "HA is enabled: ${pacemaker_and_corosync_are_running}. Replication is enabled: ${replication_is_enabled}" >>/var/www/product_update/update.log

if [[ ($pacemaker_and_corosync_are_running == "True" && $replication_is_enabled == "True") || $replication_is_enabled == "True" ]]; then
    # Returns "True" or "False"
    maintenance_mode_on=$(maintenance_is_mode_on)

    if [[ $maintenance_mode_on == "False" ]]; then
        echo "Aborting update. High availability or database replication is enabled and maintenance mode has not been turned on." >>/var/www/product_update/update.log
        echo "500 - Error: high availability or database replication is enabled. Please enable maintenance mode and try installing the update again." >/var/www/product_update/installation.progress
        sleep 5
        exit 1
    fi
fi

echo "Neither HA nor replication appear to be enabled. Proceeding with update." >>/var/www/product_update/update.log

exit 0
