#!/usr/bin/env bash

#remove mech list from sasl2 slapd.conf
cat >/etc/sasl2/slapd.conf <<EOF
pwcheck_method: saslauthd
saslauthd_path: /var/run/saslauthd/mux
EOF

# stop openldap service and remove openldap files
systemctl stop slapd

# start openldap service
systemctl start slapd

grep -o "'.*'" /etc/openldap/slapd.d/cn=config/cn=schema/cn={4}directoryobject.ldif | sed "s/'//g; s/directoryObject DESC//g" >temp_attributes.txt

number_of_attributes=$(sed -n '$=' temp_attributes.txt)

cat >full_attributelist.txt <<EOF
sAMAccountName
customAttribute1
customAttribute2
customAttribute3
customAttribute4
customAttribute5
customAttribute6
customAttribute7
customAttribute8
customAttribute9
customAttribute10
localData1
localData2
localData3
localData4
localData5
localData6
localData7
localData8
localData9
localData10
guid
cmsourceid
EOF

#attribute not yet included
grep -Fvxf temp_attributes.txt full_attributelist.txt >attributes_to_include.txt

number_of_attributes_to_include=$(sed -n '$=' attributes_to_include.txt)

if [[ -z "$number_of_attributes_to_include" ]]; then
  echo "No new attributes to add"
else
  echo "$number_of_attributes_to_include attributes to include"

  i=$(($number_of_attributes - $number_of_attributes_to_include + 9))

  # create ldif to modify olcSizeLimit value
  cat >addCustomAttributes.ldif <<EOF
dn: cn={4}directoryobject,cn=schema,cn=config
changetype: modify
add: olcAttributeTypes
EOF

  FILE="attributes_to_include.txt"
  new_attributes=$(cat $FILE)
  for attribute in $new_attributes; do
    cat >>addCustomAttributes.ldif <<EOF
olcAttributeTypes: ( 1.2.3.4.5.6.$i
  NAME '$attribute'
  SYNTAX 1.3.6.1.4.1.1466.115.121.1.15
  EQUALITY caseIgnoreMatch
  SUBSTR caseIgnoreSubstringsMatch
  SINGLE-VALUE )
EOF
    i=$((i + 1))
  done

  #ger file wth remove break line
  tr -d '\n' <\/etc\/openldap\/slapd.d\/cn=config\/cn=schema\/cn\=\{4\}directoryobject.ldif >directoryobject_ldif_no_breaks.txt

  current_atrributes_to_append=$(grep -o "MAY.*)" directoryobject_ldif_no_breaks.txt | sed 's/\b\s\b//g; s/) )//')

  cat >>addCustomAttributes.ldif <<EOF
-
delete: olcObjectClasses
olcObjectClasses: {0}( 1.2.3.4.5.6.8
  NAME 'directoryObject'
  DESC 'Directory entry item'
  SUP inetOrgPerson
  STRUCTURAL
  $current_atrributes_to_append ) )
-
add: olcObjectClasses
olcObjectClasses: {0}( 1.2.3.4.5.6.8
  NAME 'directoryObject'
  DESC 'Directory entry item'
  SUP inetOrgPerson
  STRUCTURAL
  $current_atrributes_to_append
EOF

  truncate -s -1 addCustomAttributes.ldif

  for attribute in $new_attributes; do
    echo -n "$ $attribute " >>addCustomAttributes.ldif
  done

  echo -n ") )" >>addCustomAttributes.ldif

  #run ldapmodify
  ldapadd -Y EXTERNAL -H ldapi:/// -f addCustomAttributes.ldif

  #add mech_list back into slapd.conf
  cat >/etc/sasl2/slapd.conf <<EOF
pwcheck_method: saslauthd
mech_list: plain
saslauthd_path: /var/run/saslauthd/mux
EOF

  # stop openldap service and remove openldap files
  systemctl stop slapd

  # start openldap service
  systemctl start slapd

  \rm -rf directoryobject_ldif_no_breaks.txt
  \rm -rf addCustomAttributes.ldif

fi

\rm -rf temp_attributes.txt
\rm -rf full_attributelist.txt
\rm -rf attributes_to_include.txt
