#!/usr/bin/env bash

BASE="/home/akkadianuser/packages/"
ALT_BASE="/home/akkadianuser/packages/apm_acm/"
VERSION="1.0.3"
RELEASE="5"
RPM_FILENAME="AkkadianServiceMonitor-"$VERSION"-"$RELEASE".noarch.rpm"
RPM_PATH=$BASE"$RPM_FILENAME"

ARG1=$1
NEW_INSTALL_STATUS=99

if [[ ! -f $RPM_PATH ]]; then
    # use alt path
    RPM_PATH=$ALT_BASE"$RPM_FILENAME"
fi

if [[ -f $RPM_PATH && -s $RPM_PATH ]]; then
    # run sudo install, check output with successfive greps to see if it was successfull.
    NEW_INSTALL_STATUS=$(sudo yum -y install $RPM_PATH 2>&1 | grep -w -A 2 'Installed:' | grep -q -w 'AkkadianServiceMonitor*' ; echo ${PIPESTATUS[2]})
fi

if (( $NEW_INSTALL_STATUS == 0)); then
    echo "Successfully installed ASM Version: ${VERSION}-${RELEASE}" >>/var/www/product_update/update.log
elif (( $NEW_INSTALL_STATUS == 1)); then

    # run install again and see if it's caused by nothing to do error.
    SECOND_INSTALL_STATUS=$(sudo yum -y install $RPM_PATH 2>&1 | grep -q -e 'Error: Nothing to do' ; echo ${PIPESTATUS[1]}) # TODO: use a variable for install output . save the call.

    if (($SECOND_INSTALL_STATUS == 0)); then #same version installed. Process as success or check if reinstallation
        if [[ $ARG1 == reinstall ]]; then
            REINSTALL_STATUS=$(sudo yum -y reinstall $RPM_PATH 2>&1 | grep -w -A 2 'Installed:' | grep -q -w 'AkkadianServiceMonitor*' ; echo ${PIPESTATUS[2]} )

            if (( $REINSTALL_STATUS == 0 )); then
                echo "Successfully reinstalled ASM Version: ${VERSION}-${RELEASE}" >>/var/www/product_update/update.log
            else
                 echo "Reinstallation failed for ASM Version: ${VERSION}-${RELEASE}" >>/var/www/product_update/update.log
            fi 
        else 
            echo "Skipping installation of ASM Version: ${VERSION}-${RELEASE}, Same or newer version installed." >>/var/www/product_update/update.log
        fi
        
    else
        echo "Installation failed for ASM Version: ${VERSION}-${RELEASE}" >>/var/www/product_update/update.log
    fi
elif (( $NEW_INSTALL_STATUS == 99 )); then
    echo "Installation failed for ASM Version: ${VERSION}-${RELEASE}. There was a problem finding the required installation file while searching at $RPM_PATH" >>/var/www/product_update/update.log
else
    echo "Installation failed for ASM Version: ${VERSION}-${RELEASE}" >>/var/www/product_update/update.log
fi
