from subprocess import Popen, PIPE

import click

import configparser

mapper = configparser.RawConfigParser()
mapper.read('mapper.ini')


def remove_yum_proxy_configuration():
    """Remove all proxy settings from /etc/yum.conf"""
    try:
        p = Popen(["sh", mapper.get('GeneralScripts', 'removeYumProxyOptions')],
                  stdout=PIPE, stderr=PIPE)

        for line in p.stdout:
            line = line.decode("utf-8")
            print(line)
        for line in p.stderr:
            print(line)

        click.secho('All yum proxy settings have been removed.', fg='green')
        click.pause('Please, press any key to continue...')

    except Exception as e:
        print(e)


def set_yum_proxy_configuration():
    """Add proxy address, username, and password values to yum.conf"""
    proxy_address = ''

    click.secho('Type quit to cancel proxy configuration.', fg="green")
    click.secho('Valid proxy protocols are http, https, ftp, socks4, socks4a, socks5, and socks5h.', fg="green")

    proxy_uri_is_valid = False

    while not proxy_uri_is_valid and proxy_address != "quit":
        proxy_address = click.prompt('Enter the proxy URI, e.g., https://www.example.com:3128')

        proxy_uri_is_valid = (
            proxy_address.startswith('http')
            or proxy_address.startswith('https')
            or proxy_address.startswith('ftp')
            or proxy_address.startswith('socks4')
            or proxy_address.startswith('socks4a')
            or proxy_address.startswith('socks5')
            or proxy_address.startswith('socks5h')
        )

    if proxy_address == "q" or proxy_address == "quit":
        return

    proxy_username = click.prompt('Enter the proxy username', default="", show_default=False)

    if proxy_username == "quit":
        return

    proxy_password = click.prompt('Enter the proxy password', default="", show_default=False)

    if proxy_password == "quit":
        return

    try:
        p = Popen(
            ["sh", mapper.get('GeneralScripts', 'setYumProxyOptions'), proxy_address, proxy_username, proxy_password],
            stdout=PIPE, stderr=PIPE)

        click.secho('Yum proxy settings have been configured.', fg='green')
        click.pause('Please, press any key to continue...')

    except Exception as e:
        print(e)


def view_yum_proxy_configuration():
    """View the current values assigned to proxy, proxy_username, and proxy_password in /etc/yum.conf"""
    try:
        proxy_address = "N/A"
        proxy_password = "N/A"
        proxy_username = "N/A"

        yum_config = open("/etc/yum.conf", "r")
        yum_config_lines = yum_config.readlines()
        yum_config.close()

        for config_line in yum_config_lines:
            equals_sign_index = config_line.find("=")

            if equals_sign_index == -1 or equals_sign_index == (len(config_line) - 1):
                continue

            if "proxy_username" in config_line:
                proxy_username = config_line[equals_sign_index:].lstrip('=').strip()
            elif "proxy_password" in config_line:
                proxy_password = config_line[equals_sign_index:].lstrip('=').strip()
            elif "proxy" in config_line:
                proxy_address = config_line[equals_sign_index:].lstrip('=').strip()

        click.secho("Current Yum Proxy Settings", fg="green")
        click.secho(f"Proxy Address: {proxy_address}", fg="green")
        click.secho(f"Proxy Username: {proxy_username}", fg="green")
        click.secho(f"Proxy Password: {proxy_password}", fg="green")
        click.pause('Please, press any key to continue...')

    except Exception as e:
        print(e)
